/*
	all routes pertaining to oauth stuff, login logout and token, passed the app object VIA app.js
*/
module.exports = function(router) {


	/*
		home page, check out views/index.jade out to find the template
	*/
	router.get('/', function(req, res, next) {	
		sess = req.session;
		if(sess.loggedIn) {
			res.redirect('/files');
			return;
		}

		var cloud =  req.app.locals.cloud(req);		

	  	res.render('index', { cloud: cloud });
	});


	/*
		exchanges an auth code for access tokens, and redirects a user if successful
	*/
	router.get('/token', function(req, res, next) {	

		sess = req.session;
		if(sess.loggedIn) {
			res.redirect('/files');
			return;
		}

		console.log("/token request");

		var cloud =  req.app.locals.cloud(req);		

		// user clicked deny, so we got redirected with an error code
		if(req.query.error) {
			res.render('index', { 
				cloud: cloud, 
				error: "User denied access to this app",
			});
			return;
		}

		// try to exchange the auth code for an access token, otherwise prompt a user with some error
		var tokens = cloud.token({ 
			code: req.query.code, 
			success: function(resp) {
				sess.loggedIn = true;
				sess.oauth = resp.body;
				res.redirect('/files');

			},
			failure: function(resp) {
				res.render('index', { 
					cloud: cloud, 
					error: "Invalid code",
				});
			}
		});
	});

	/* 
		nukes the session and redirects to the home page
	*/
	router.get('/logout', function(req, res, next) {	
		req.session.destroy();
		res.redirect('/');
	});

}