/*
	all routes pertaining to playlist stuff, playlist detail, playlists, and gettting playlist content
*/
module.exports = function(router) {

	/*
		fetches all playlists for this user 
	*/
	router.get('/playlists', function(req, res, next) {	
		sess=req.session;
		console.log(sess);	
		if(!sess.loggedIn) {
			res.redirect('/');
			return;
		}
		
		var cloud =  req.app.locals.cloud(req);		

		cloud.playlists({
			success: function(resp) {
				//dig into the json and pass the playlists to the template for easy rendering, 
				//that's the deeply nested structure we get back from the rest API
				res.render('playlists', { 
					playlists: resp.body.playlistDefinitions.playlistDefinition 
				});
			},
			failure: function(resp) {
				res.render('playlists', { 
					error: "Error calling playlists" 
				});
			}
		});
	}); 

	/*
		Gets details about a playlist. This router will call two cloud SDK methods, 'playlist' to get metadata about the playlist
		and 'playlistItems' to fetch all the items inside a playlist
	*/ 
	router.get('/playlists/:playlistId', function(req, res, next) {	
		sess = req.session;
		if(!sess.loggedIn) {
			res.redirect('/');
			return;
		}
		
		var cloud =  req.app.locals.cloud(req);		
		
		//if either of the two API calls fail we want to show an error state, so we pass this into the failure callback
		var failure = function(resp) {
			res.render('playlistInfo', { error: "Error calling playlists" });
			return;		
		}

		//first get the metadata about the playlist
		cloud.playlist({					
			uid: req.params.playlistId, //passed as part of the path 
			success: function(resp) {
				var playlistInfo = resp.body.playlistDefinition;

				//now call playlistItems and render the template
				cloud.playlistItems({
						uid: req.params.playlistId,
						success: function(resp) {		
							//we have all the data we need, just pass it to the template and let it render it				
							res.render('playlistInfo', { 
								cloud: cloud, 
								playlistInfo: playlistInfo,
								uid: req.params.playlistId,
								playlist: resp.body.playlist 
							});
						}, 
						failure: failure
				});
			},
			failure: failure
		});
	});	
}