var fs = require('fs'); 
var tmp = require('tmp'); // since users will be temporarily saving files on our server we want to make sure those files are managed
var crypto = require('crypto'); // we use this to calculate the sha256 checksum of the file



/*
	this is the most complicated router, it's responsible for uploading a file using the Cloud SDK
*/
module.exports = function(router){
	
	router.post('/fileupload', function(req, res) {
		sess=req.session;
		if(!sess.loggedIn) {
			res.redirect('/');
			return;
		}
		var cloud =  req.app.locals.cloud(req);		

	    // give the file to busboy in order to start the download of the file, express 4 has no handlers for this stuff anymore 
	    // so we're forced to use busboy: http://stackoverflow.com/questions/23114374/file-uploading-with-express-4-0-req-files-undefined
	    req.pipe(req.busboy);

	    req.busboy.on('file', function (fieldname, file, filename) {
	        if(!filename) {
	          res.redirect('back');
	          return;
	        }

	        // first create a temporary file
	       	var fstream = fs.createWriteStream(tmp.tmpNameSync());
	       	var tempFilePath = fstream.path;
	        var hash = crypto.createHash('sha256');
	        var computedHash;
	        var fileSizeInBytes;

			hash.setEncoding('hex');
	        
	        // calculate the hash in a stream, one chunk at a time, then read the result when we're all done
	        hash.on('finish', function () {
				hash.end();      	
				computedHash = hash.read();
	            finished();
	        });

	        // save the file to the file system and calculate the size 
	        fstream.on('finish', function () {
				var stats = fs.statSync(tempFilePath);
				fileSizeInBytes = stats["size"];
				finished();
	        });

	        // since we are going to pipe the stream to both a hash and file saving routine, we need to wait until both are done
	        // to proceed with the upload
	        var finished = function() {
	        	if(!fileSizeInBytes || !computedHash) 
	        		return false;
	        	var filestream = fs.createReadStream(tempFilePath);
	        	console.log("file stream", filestream);
	        		        	
	        	// we have everything we need, let's call the upload facade with the file stream, and the necessary metadata
	        	// note that the upload facade handles both chunked and unchunked uploads auotmatically, if a file is over 100mb it will 
	        	// use the chunked upload, otherwise it will use unchunked
				cloud.upload({
		    		checksum: computedHash, 
		    		concurrentChunks: 8, //8 chunks in parallel at a time, default is 5. NOTE this is only used when the file is > 100mb and chunked uploads are used
					chunkSize: 2097152* 3, //6mb, default is 2mb.  NOTE this is only used when the file is > 100mb and chunked uploads are used
		    		name: filename,
		    		path: '/VZMOBILE',
		    		size: fileSizeInBytes,
		    		fs: filestream, // pass the file stream to the cloud SDK, it will stream the contents to the server
		    		success: function(success) {
		    			//the file has been upladed, we can grab the new file's metadata here 
		    			console.log(success.body.file);
		    			res.redirect('back');
		    		}, 
		    		failure: function(failure) {
		    			console.log("failed", (failure.body));
		    			res.redirect('back');
		    		}
		    	}); 

	        }

	        // we need to pipe the file stream to the hasher, and the temporary file on disk so we dont lose it
	        // since these files could be GBs in size, everything is done VIA streams
	        file.pipe(hash);
	        file.pipe(fstream);

	    });
	});



}
